@@ -5,7 +5,8 @@ Boost Software License - Version 1.0.  See the accompanying
 LICENSE file or <http://www.boost.org/LICENSE_1_0.txt>
 */
 
-#include "cell.h"
+#define __STDC_FORMAT_MACROS 1
+#define _USE_MATH_DEFINES
 
 #include <float.h>
 #include <inttypes.h>
@@ -14,12 +15,12 @@ LICENSE file or <http://www.boost.org/LICENSE_1_0.txt>
 #include <string.h>
 #include <time.h>
 
-#include "allocator.h"
-#include "cell.h"
-#include "rawcell.h"
-#include "sort.h"
-#include "utils.h"
-#include "vec.h"
+#include <gdstk/allocator.hpp>
+#include <gdstk/cell.hpp>
+#include <gdstk/rawcell.hpp>
+#include <gdstk/sort.hpp>
+#include <gdstk/utils.hpp>
+#include <gdstk/vec.hpp>
 
 namespace gdstk {
 
@@ -595,6 +596,29 @@ void Cell::flatten(bool apply_repetitions, Array<Reference*>& result) {
     }
 }
 
+void Cell::remap_tags(const TagMap& map) {
+    for (uint64_t i = 0; i < polygon_array.count; i++) {
+        Polygon* polygon = polygon_array[i];
+        polygon->tag = map.get(polygon->tag);
+    }
+    for (uint64_t i = 0; i < flexpath_array.count; i++) {
+        FlexPath* path = flexpath_array[i];
+        for (uint64_t j = 0; i < path->num_elements; j++) {
+            path->elements[j].tag = map.get(path->elements[j].tag);
+        }
+    }
+    for (uint64_t i = 0; i < robustpath_array.count; i++) {
+        RobustPath* path = robustpath_array[i];
+        for (uint64_t j = 0; i < path->num_elements; j++) {
+            path->elements[j].tag = map.get(path->elements[j].tag);
+        }
+    }
+    for (uint64_t i = 0; i < label_array.count; i++) {
+        Label* label = label_array[i];
+        label->tag = map.get(label->tag);
+    }
+}
+
 void Cell::get_dependencies(bool recursive, Map<Cell*>& result) const {
     Reference** reference = reference_array.items;
     for (uint64_t i = 0; i < reference_array.count; i++, reference++) {
@@ -880,7 +904,7 @@ ErrorCode Cell::write_svg(const char* filename, double scaling, uint32_t precisi
 
     FILE* out = fopen(filename, "w");
     if (out == NULL) {
-        fputs("[GDSTK] Unable to open file for SVG output.\n", stderr);
+        if (error_logger) fputs("[GDSTK] Unable to open file for SVG output.\n", error_logger);
         return ErrorCode::OutputFileOpenError;
     }
 
